@@ -33,6 +33,7 @@ from scim import Attribute
 import PYParser
 import PYSQLiteDB
 import SpecialTable
+import SpecialPhrase
 import PYUtil
 import PYDict
 import scim.ascii as ascii
@@ -67,6 +68,7 @@ class PinYinEngine (IMEngine):
 	_pydb = PYSQLiteDB.PYSQLiteDB (user_db = "user.db")
 
 	# create special table
+	_special_phrase = SpecialPhrase.SpecialPhrase ()
 	_special_table = SpecialTable.SpecialTable ()
 	
 	# shuang pin 
@@ -92,14 +94,15 @@ class PinYinEngine (IMEngine):
 	# lookup table page size
 	_page_size = 5
 
-	# press [i] or [v] to temp English mode
-	_ivu_to_temp = True
+	# press [u] or [v] to temp English mode
+	_uv_to_temp = True
 
 
 	def __init__ (self, factory, config, encoding, id):
 		IMEngine.__init__ (self, factory, config, encoding, id)
 		self._config = config
 		self._lookup_table = scim.LookupTable (PinYinEngine._page_size)
+		self._lookup_table.fix_page_size (True)
 		
 		self._py_parser = PYParser.PinYinParser ()
 		self._user_input = UserInput (self._py_parser)
@@ -197,14 +200,14 @@ class PinYinEngine (IMEngine):
 				return
 			self._english_candidates = []
 			string = "".join (self._user_input.get_chars ())
-			if string[0] in u"iuv":
+			if string[0] in u"uv":
 				string = string [1:]
 			words = string.split ()
 			if not words:
 				return
 			word = words[-1]
 			
-			if len (words) == 1 and word[0:1] in u"iuv":
+			if len (words) == 1 and word[0:1] in u"uv":
 				word = word[1:]
 
 			if not word.isalpha ():
@@ -216,6 +219,12 @@ class PinYinEngine (IMEngine):
 			is_same  = lambda x : x[0].isupper () == word[0].isupper ()
 			self._english_candidates = filter (is_same, candidates)[:10]
 			return
+
+		if self._i_mode:
+			chars = self._user_input.get_chars()[1:]
+			self._candidates = self._special_phrase.lookup (u"".join (chars))
+			self._candidates += self._special_table.lookup (u"".join (chars))
+			return
 		
 		self._preedit_phrases.clear ()
 		
@@ -225,7 +234,7 @@ class PinYinEngine (IMEngine):
 			return
 
 		if len (self._committed_phrases) == 0:
-			self._special_candidates = self._special_table.lookup (u"".join (self._user_input.get_chars ()))
+			self._special_candidates = self._special_phrase.lookup (u"".join (self._user_input.get_chars ()))
 		else:
 			self._special_candidates = []
 			
@@ -246,10 +255,30 @@ class PinYinEngine (IMEngine):
 			count += candidate[PYSQLiteDB.YLEN]
 
 	def _update_ui (self):
-
+		if self._i_mode:
+			preedit_string = u"".join (self._user_input.get_chars ())
+			self.update_preedit_string (preedit_string)
+			self.update_preedit_caret (len (preedit_string))
+			self.show_preedit_string ()
+			
+			self.update_aux_string (u"")
+			self.hide_aux_string ()
+			
+			self._lookup_table.clear ()
+			self._lookup_table.show_cursor (True)
+			if not self._candidates:
+				self.hide_lookup_table ()
+			else:
+				for c in self._candidates:
+					attrs = []
+					self._lookup_table.append_candidate (c, attrs)
+				self.update_lookup_table (self._lookup_table)
+				self.show_lookup_table ()
+			return 
+			
 		if self._temp_english_mode:
 			preedit_string = u"".join (self._user_input.get_chars ())
-			if preedit_string [0:1] in (u"i", u"v", "u"):
+			if preedit_string [0:1] in (u"v", u"u"):
 				preedit_string = " " + preedit_string[1:]
 			else:
 				preedit_string = " " + preedit_string
@@ -479,6 +508,8 @@ class PinYinEngine (IMEngine):
 		else:
 			if self._temp_english_mode:
 				return self._temp_english_mode_process_key_event (key)
+			elif self._i_mode:
+				return self._i_mode_process_key_event (key)
 			else:
 				return self._chinese_mode_process_key_event (key)
 
@@ -538,11 +569,66 @@ class PinYinEngine (IMEngine):
 		
 		# should not reach there
 		return False
-	
+
+	def _i_mode_process_key_event (self, key):
+		if key.code in (KeyCode.KEY_Return, KeyCode.KEY_KP_Enter):
+			commit_string = u"".join (self._user_input.get_chars ())
+			self.commit_string (commit_string)
+			return True
+		elif key.code == KeyCode.KEY_BackSpace and len (self._user_input) != 0:
+			self._user_input.pop ()
+			if len (self._user_input) == 0:
+				self._i_mode = False
+			self._update ()
+			return True
+		elif key.code == KeyCode.KEY_Escape:
+			self._user_input.clear ()
+			self._i_mode = False
+			self._update ()
+			return True
+		elif key.code >= KeyCode.KEY_1 and key.code <= KeyCode.KEY_9:
+			if not self._candidates:
+				return True
+			index = key.code - KeyCode.KEY_1
+			if index >= PinYinEngine._page_size:
+				return True
+			index += self._lookup_table.get_current_page_start ()
+			if index >= len (self._candidates):
+				return True
+			self.commit_string (self._candidates[index])
+			return True
+		elif key.code in (KeyCode.KEY_KP_Space, KeyCode.KEY_space):
+			if not self._candidates:
+				return True
+			index = self._lookup_table.get_cursor_pos ()
+			if index >= len (self._candidates):
+				return True
+			self.commit_string (self._candidates[index])
+			return True
+		elif key.code == KeyCode.KEY_Down:
+			self.lookup_table_cursor_down ()
+			return True
+		elif key.code == KeyCode.KEY_Up:
+			self.lookup_table_cursor_up ()
+			return True
+		elif key.code in (KeyCode.KEY_equal, KeyCode.KEY_period, KeyCode.KEY_Page_Down) and self._candidates:
+			self.lookup_table_page_down ()
+			return True
+		elif key.code in (KeyCode.KEY_minus, KeyCode.KEY_comma, KeyCode.KEY_Page_Up) and self._candidates:
+			self.lookup_table_page_up ()
+			return True
+
+		if key.code >= 128:
+			return True
+		self._user_input.append (unichr (key.code))
+		self._update ()
+		
+		return True
+
 	def _temp_english_mode_process_key_event (self, key):
 		if key.code in (KeyCode.KEY_Return, KeyCode.KEY_KP_Enter):
 			commit_string = u"".join (self._user_input.get_chars ())
-			if commit_string[0] in (u"v", u"i", u"u"):
+			if commit_string[0] in (u"v", u"u"):
 				commit_string = commit_string[1:]
 			self.commit_string (commit_string)
 			return True
@@ -570,6 +656,13 @@ class PinYinEngine (IMEngine):
 				self._update ()
 				return True
 			return False
+		elif key.code in (KeyCode.KEY_Page_Down, ) and self._english_candidates:
+			self.lookup_table_page_down ()
+			return True
+		elif key.code in (KeyCode.KEY_Page_Up, ) and self._english_candidates:
+			self.lookup_table_page_up ()
+			return True
+
 		if key.code >= 128:
 			return True
 
@@ -668,9 +761,9 @@ class PinYinEngine (IMEngine):
 			self._pydb.commit_char (char, pinyin_id, shengmu_id)
 			self.commit_string (char)
 			return True
-		elif PinYinEngine._ivu_to_temp and not PinYinEngine._shuangpin \
+		elif PinYinEngine._uv_to_temp and not PinYinEngine._shuangpin \
 		     and len (self._user_input) == 0 \
-		     and key.code in (KeyCode.KEY_v, KeyCode.KEY_i, KeyCode.KEY_u):
+		     and key.code in (KeyCode.KEY_v, KeyCode.KEY_u):
 			self._user_input.append (unichr (key.code))
 			self._temp_english_mode = True
 			self._update ()
@@ -680,6 +773,13 @@ class PinYinEngine (IMEngine):
 			self._temp_english_mode = True
 			self._update ()
 			return True
+		elif not PinYinEngine._shuangpin \
+			 and len (self._user_input) == 0 \
+			 and key.code == KeyCode.KEY_i:
+			self._user_input.append (unichr (key.code))
+			self._i_mode = True
+			self._update ()
+			return True
 		elif (key.code >= KeyCode.KEY_a and key.code <= KeyCode.KEY_z) or \
 			(key.code == KeyCode.KEY_apostrophe and len (self._user_input) != 0) or \
 			(key.code == KeyCode.KEY_semicolon and len (self._user_input) != 0 and PinYinEngine._shuangpin) :
@@ -753,6 +853,7 @@ class PinYinEngine (IMEngine):
 
 	def commit_string (self, string):
 		self._temp_english_mode = False
+		self._i_mode = False
 		self._candidates = []
 		self._english_candidates = []
 		self._cursor = 0
@@ -807,6 +908,7 @@ class PinYinEngine (IMEngine):
 
 	def reset (self):
 		self._temp_english_mode = False
+		self._i_mode = False
 		self._user_input.clear ()
 		self._committed_phrases.clear ()
 		self._committed_special_phrase = u""
@@ -1048,7 +1150,7 @@ class Factory (IMEngineFactory):
 			config.read ("/IMEngine/Python/PinYin/EnglishPhraseColor", PinYinEngine._english_phrase_color)
 		PinYinEngine._error_eng_phrase_color = \
 			config.read ("/IMEngine/Python/PinYin/ErrorEnglishPhraseColor", PinYinEngine._error_eng_phrase_color)
-		PinYinEngine._ivu_to_temp = \
-			config.read ("/IMEngine/Python/PinYin/IVUToTemp", PinYinEngine._ivu_to_temp)
+		PinYinEngine._uv_to_temp = \
+			config.read ("/IMEngine/Python/PinYin/UVToTemp", PinYinEngine._uv_to_temp)
 
 
