@@ -34,15 +34,17 @@ from PYDict import *
 from gettext import dgettext
 from ZhengJuDB import *
 import scim.ascii as ascii
+from sets import Set
 import popen2
 
 _ = lambda a : dgettext ("scim-python", a)
 RGB = lambda r, g, b : (((r & 0xff) << 16) | ((g & 0xff) << 8) | (b & 0xff))
-
 IMEngine = scim.IMEngine
 IMEngineFactory = scim.IMEngineFactory
 
 (YLEN, Y0, Y1, Y2, Y3, YX, PHRASE, ADJ_FREQ) = range (0, 8)
+candidate_sort = lambda x,y: cmp(y[YLEN],x[YLEN]) if x[YLEN] != y[YLEN] else cmp(y[ADJ_FREQ],x[ADJ_FREQ])
+
 class InputException(Exception):
 	def __init__ (self):
 		Exception.__init__(self)
@@ -53,21 +55,16 @@ class PinYinWord:
 		self._pinyin_id = None
 		self.manual = None
 		self.char = ""
+		self.spliter = ""
 		if pinyin:
 			self.set_pinyin(pinyin)
 		else:
-			self.shengmu = shengmu
-			self.yunmu = yunmu
-			if self.get_pinyin() in PINYIN_LIST:
-				self._pinyin_id = PINYIN_DICT [self.get_pinyin()]
-				self._sheng_mu_id = SHENGMU_DICT [self.get_shengmu()]
-			else:
-				self._sheng_mu_id = SHENGMU_DICT [self.get_shengmu()]
+			self.set_pinyin(shengmu + yunmu)
 	def set_pinyin(self,pinyin):
 		if pinyin[0] == "'":
-			self.shengmu = "'"
-			self.yunmu = pinyin[1:]			
-		elif pinyin[:2] in SHENGMU_LIST:
+			self.spliter = "'"
+			pinyin = pinyin[1:]
+		if pinyin[:2] in SHENGMU_LIST:
 			self.shengmu = pinyin[:2]
 			self.yunmu = pinyin[2:]
 		elif pinyin[:1] in SHENGMU_LIST:
@@ -81,7 +78,20 @@ class PinYinWord:
 			self._sheng_mu_id = SHENGMU_DICT [self.get_shengmu()]
 		else:
 			self._sheng_mu_id = SHENGMU_DICT [self.get_shengmu()]
-
+	def mohuyin(self):
+		pinyin = ID_PINYIN_DICT[self.real_pinyin_id]
+		if pinyin[:2] in SHENGMU_LIST:
+			self.shengmu = pinyin[:2]
+			yunmu = pinyin[2:]
+		elif pinyin[:1] in SHENGMU_LIST:
+			self.shengmu = pinyin[:1]
+			yunmu = pinyin[1:]
+		else:
+			self.shengmu = ""
+			yunmu = pinyin
+		if self.yunmu != "":
+			 self.yunmu = yunmu
+		self.set_pinyin(self.get_pinyin())
 	def get_sheng_mu_id (self):
 		return self._sheng_mu_id
 
@@ -92,13 +102,8 @@ class PinYinWord:
 		self.set_pinyin(ID_PINYIN_DICT[id])
 		
 	def get_shengmu (self):
-		if self.shengmu == "'":
-			return ""
 		return self.shengmu
 	
-	def set_shengmu (self,shengmu):
-		self.shengmu = shengmu
-
 	def set_yunmu( self,yunmu):
 		self.yunmu = yunmu
 		if(yunmu != ""):
@@ -110,12 +115,10 @@ class PinYinWord:
 		self.char = char
 
 	def get_pinyin (self):
-		if self.shengmu == "'":
-			return self.yunmu
 		return self.shengmu + self.yunmu
 
 	def get_screen_pinyin (self):
-		return self.shengmu + self.yunmu
+		return self.spliter + self.shengmu + self.yunmu
 
 	def __str__ (self):
 		return self.get_pinyin()
@@ -131,18 +134,20 @@ class Editor:
 			config = PseudoConfig()
 		if Editor.database == None:
 			Editor.database = ZhengJuDB(config)
+		self.lookup_table = scim.LookupTable (9)
+		self.lookup_table.fix_page_size(True)
 		self.clear()
 		self.config = config
 		self.load_config(config)
 
 	def clear(self):
 		self.cursor = 0
-		self.candidate_cursor = None
 		self.wordlist = []
 		self.pinyinlist = []
 		self.candidates = []
 		self.predict = []
-		self.page = 0
+		self.lookup_table.clear()
+		self.lookup_table.show_cursor(False)
 		Editor.database.clear_cache()
 	def load_config(self, config):
 		Editor.database.load_config(config)
@@ -151,7 +156,39 @@ class Editor:
 		self.adjustfreq = config.read ("/IMEngine/Python/ZhengJu/AdjustWordFreq", True)
 		self.logconverror = config.read ("/IMEngine/Python/ZhengJu/LogConvError", True)
 		self.splitpinyin = config.read ("/IMEngine/Python/ZhengJu/SplitPinyin", True)
-
+		self.enable_mohuyin = config.read ("/IMEngine/Python/ZhengJu/FuzzyPinyin", False)
+		self.mohuyin_s_sh = config.read ("/IMEngine/Python/ZhengJu/FuzzyS_Sh", True)
+		self.mohuyin_c_ch = config.read ("/IMEngine/Python/ZhengJu/FuzzyC_Ch", True)
+		self.mohuyin_z_zh = config.read ("/IMEngine/Python/ZhengJu/FuzzyZ_Zh", True)
+		self.mohuyin_l_n = config.read ("/IMEngine/Python/ZhengJu/FuzzyL_N", True)
+		self.mohuyin_in_ing = config.read ("/IMEngine/Python/ZhengJu/FuzzyIn_Ing", True)
+		self.mohuyin_en_eng = config.read ("/IMEngine/Python/ZhengJu/FuzzyEn_Eng", True)
+		self.mohuyin_an_ang = config.read ("/IMEngine/Python/ZhengJu/FuzzyAn_Ang", True)
+		self.build_mohuyin()
+	def build_mohuyin(self):
+		self.shengmu_mohu = {}
+		if self.mohuyin_s_sh:
+			self.shengmu_mohu["s"]= MOHU_SHENGMU["s"]
+			self.shengmu_mohu["sh"]= MOHU_SHENGMU["sh"]
+		if self.mohuyin_z_zh:
+			self.shengmu_mohu["z"]= MOHU_SHENGMU["z"]
+			self.shengmu_mohu["zh"]= MOHU_SHENGMU["zh"]
+		if self.mohuyin_c_ch:
+			self.shengmu_mohu["c"]= MOHU_SHENGMU["c"]
+			self.shengmu_mohu["ch"]= MOHU_SHENGMU["ch"]
+		if self.mohuyin_l_n:
+			self.shengmu_mohu["l"]= MOHU_SHENGMU["l"]
+			self.shengmu_mohu["n"]= MOHU_SHENGMU["n"]
+		self.yunmu_mohu = {}
+		if self.mohuyin_an_ang:
+			self.yunmu_mohu["an"] = MOHU_YUNMU["an"]
+			self.yunmu_mohu["ang"] = MOHU_YUNMU["ang"]
+		if self.mohuyin_en_eng:
+			self.yunmu_mohu["en"] = MOHU_YUNMU["en"]
+			self.yunmu_mohu["eng"] = MOHU_YUNMU["eng"]
+		if self.mohuyin_in_ing:
+			self.yunmu_mohu["in"] = MOHU_YUNMU["in"]
+			self.yunmu_mohu["in"] = MOHU_YUNMU["ing"]
 	def current (self):
 		if self.pinyinlist:
 			return self.pinyinlist[-1]
@@ -201,9 +238,13 @@ class Editor:
 			if i<4:
 				#~ self.pinyinlist[i].set_pinyin_id (candidate[i+1])
 				self.pinyinlist[i].real_pinyin_id = candidate[i+1]
+				if self.enable_mohuyin:
+					self.pinyinlist[i].mohuyin()
 			else:
 				py = candidate[YX].split("'")
 				self.pinyinlist[i].real_pinyin_id = PINYIN_DICT[py[i-4]]
+				if self.enable_mohuyin:
+					self.pinyinlist[i].mohuyin()
 				#~ self.pinyinlist[i].set_pinyin ([py[i-4]])
 			self.pinyinlist[i].manual = manual
 		self.wordlist[self.cursor:self.cursor] = self.pinyinlist[:length]
@@ -252,17 +293,18 @@ class Editor:
 			self.pinyin_select(self.predict[0])
 			self.update_predict()
 		self.update_candidates()
-	
+
 	def auto_convert_quanpin(self):
-		self.update_predict()
+		#~ self.update_predict()
 		p = self.pinyinlist[-1].get_pinyin()
-		for t in PINYIN_LIST:
-			if p != t and t[:len(p)] == p :
-				while self.predict_len(self.predict[:2]) + 1 < len (self.pinyinlist):
-					self.pinyin_select(self.predict[0])
-					self.update_predict()
-				self.update_candidates()
-				return
+		if p not in SHENGMU_LIST and p not in PINYIN_PARTIAL_LIST:
+			for t in PINYIN_LIST:
+				if p != t and t[:len(p)] == p :
+					while self.predict_len(self.predict[:2]) + 1 < len (self.pinyinlist):
+						self.pinyin_select(self.predict[0])
+						self.update_predict()
+					self.update_candidates()
+					return
 		while self.predict_len(self.predict[:2]) < len (self.pinyinlist):
 			self.pinyin_select(self.predict[0])	
 			self.update_predict()
@@ -275,7 +317,7 @@ class Editor:
 
 	def update_predict (self):
 		if self.pinyinlist:
-			self.predict = self.get_predict_pinyinlist (self.pinyinlist)
+			self.predict = self.get_predict_pinyinlist (self.pinyinlist) 
 		else:
 			self.predict = []
 
@@ -304,8 +346,36 @@ class Editor:
 				pinyin.real_pinyin_id=pinyin._pinyin_id
 				self.wordlist.append(pinyin)
 			phrase = phrase[result[YLEN]:]
-
 	def split_phrase (self, string):
+		start = 0
+		phrase_list = []
+		while start < len(self.wordlist)-1:
+			candidate = None
+			if len(self.wordlist) - start >= 3:
+				phrase = Editor.database.select_words_by_pinyin_list_all (self.wordlist[start:start + 3])
+				#~ print len(phrase)
+				for i in phrase:
+					if i[PHRASE] == string[start:start + len(i[PHRASE]) ]:
+						if not candidate or candidate[PHRASE] < i[PHRASE]:
+							candidate = i
+				if candidate == None:
+					phrase = Editor.database.select_words_by_phrase (self.wordlist[start:start+2])
+					if phrase:
+						candidate = phrase[0]			
+			else:
+				phrase = Editor.database.select_words_by_phrase (self.wordlist[start:start+2])
+				if phrase:
+					candidate = phrase[0]
+			if candidate == None:
+				phrase_list.append ( (start, 1, string[start]))
+				start += 1
+			else:
+				phrase_list.append ( (start, len(candidate[PHRASE]), candidate[PHRASE]))
+				start += len(candidate[PHRASE])
+		if start < len (self.wordlist):
+			phrase_list.append ((start,1,string[-1]))
+		return phrase_list
+	def split_phrasev2 (self, string):
 		start = 0
 		phrase_list = []
 		while start < len(self.wordlist)-1:
@@ -378,9 +448,9 @@ class Editor:
 		Editor.database.adjust_phrase_freq (p)
 				
 	def delete_phrase(self, n):
-		if self.page + n >= len (self.candidates):
+		if n >= self.lookup_table.get_current_page_size():
 			raise InputException()
-		candidate = self.candidates[self.page + n]
+		candidate = self.candidates[self.lookup_table.get_current_page_start() + n]
 		if candidate[ADJ_FREQ] == 0 or \
 			candidate[ADJ_FREQ]%USER_PHRASE and candidate[ADJ_FREQ]%USER_WORD:
 			raise InputException()
@@ -388,9 +458,7 @@ class Editor:
 		self.update ()
 	
 	def delete_cursor_phrase(self):
-		if self.candidate_cursor == None:
-			raise InputException()
-		candidate = self.candidates[self.candidate_cursor]
+		candidate = self.candidates[self.lookup_table.get_cursor_pos() ]
 		if candidate[ADJ_FREQ] == 0 or \
 			candidate[ADJ_FREQ]%USER_PHRASE and candidate[ADJ_FREQ]%USER_WORD:
 			self.candidate_cursor = None
@@ -458,11 +526,15 @@ class Editor:
 	
 	def freq_alg(self, phrase1, phrase2):
 		freq = 0
-		if len(phrase1[PHRASE]) < 4:
+		if len(phrase1[PHRASE]) == 1:
+			freq += phrase1[ADJ_FREQ] * 10
+		elif len(phrase1[PHRASE]) < 4:
 			freq += phrase1[ADJ_FREQ] * len(phrase1[PHRASE])
 		else:
 			freq += phrase1[ADJ_FREQ] * pow( len(phrase1[PHRASE]) , 2)
-		if len(phrase2[PHRASE]) < 4:
+		if len(phrase2[PHRASE]) == 1:
+			freq += phrase2[ADJ_FREQ] * 10
+		elif len(phrase2[PHRASE]) < 4:
 			freq += phrase2[ADJ_FREQ] * len(phrase2[PHRASE])
 		else:
 			freq += phrase2[ADJ_FREQ] * pow( len(phrase2[PHRASE]) , 2)
@@ -474,20 +546,19 @@ class Editor:
 
 
 	def get_predict_pinyinlist (self, pinyinlist):
-		longest = Editor.database.get_longest_phrase_length (pinyinlist)
-		if longest >= len(pinyinlist):
-			candidates = Editor.database.select_words_by_pinyin_list (pinyinlist)
+		#~ print "Dd", u" ".join( i.get_screen_pinyin() for i in pinyinlist), len(pinyinlist)
+		candidates = Editor.database.select_words_by_pinyin_list (pinyinlist)
+		if candidates:
+			#~ print "phrase1",candidates[0][PHRASE],candidates[0][ADJ_FREQ]
+			return [candidates[0]]
+		else:
+			candidates = Editor.database.select_words_by_pinyin_list_all(pinyinlist)
 			if candidates:
-				#~ print "phrase1",candidates[0][PHRASE],candidates[0][ADJ_FREQ]
-				return [candidates[0]]
-			else:
-				candidates = Editor.database.select_words_by_pinyin_list_all(pinyinlist)
-				if candidates:
-					#~ print candidates[0][PHRASE]
-					p = list (candidates[0]);
-					p[YLEN] = len (pinyinlist)
-					p[PHRASE] = p[PHRASE][:p[YLEN]]
-					return [p]
+				#~ print candidates[0][PHRASE]
+				p = list (candidates[0]);
+				p[YLEN] = len (pinyinlist)
+				p[PHRASE] = p[PHRASE][:p[YLEN]]
+				return [p]
 		max_freq = 0
 		predict = []
 		for length in range(len (pinyinlist), 1, -1):
@@ -518,6 +589,17 @@ class Editor:
 			if predict:
 				break
 		if self.predict_len(predict) < len (pinyinlist):
+			#~ return
+			#~ for i in range(1, predict[0][YLEN]):
+				#~ candidates =  Editor.database.select_words_by_pinyin_list(pinyinlist[:i])
+				#~ if candidates and candidates[0][PHRASE] == predict[0][PHRASE][:i]:
+					#~ print "try", i, candidates[0][PHRASE]
+					#~ temp = self.get_predict_pinyinlist(pinyinlist[i:self.predict_len(predict)])
+					#~ print "resule", temp[0][PHRASE] +  temp[1][PHRASE]
+					#~ print "match", predict[0][PHRASE][i:] + predict[1][PHRASE]
+					#~ if predict[0][PHRASE][i:] + predict[1][PHRASE] == (temp[0][PHRASE] + temp[1][PHRASE]):
+						#~ print "go", candidates[0][PHRASE]
+						#~ return [candidates[0]] + temp
 			return predict + self.get_predict_pinyinlist(pinyinlist[self.predict_len(predict):])
 		else:
 			return predict
@@ -525,26 +607,24 @@ class Editor:
 	def get_predict (self, pinyinlist):
 		if not pinyinlist:
 			return []
-		longest = Editor.database.get_longest_phrase_length (pinyinlist)
-		if longest >= len (pinyinlist):
-			candidates = Editor.database.select_words_by_pinyin_list(pinyinlist)
+		candidates = Editor.database.select_words_by_pinyin_list(pinyinlist)
+		if candidates:
+			#~ print "phrase1",candidates[0][PHRASE],candidates[0][ADJ_FREQ]
+			return [candidates[0]]
+		else:
+			candidates = Editor.database.select_words_by_pinyin_list_all(pinyinlist)
 			if candidates:
-				#~ print "phrase1",candidates[0][PHRASE],candidates[0][ADJ_FREQ]
-				return [candidates[0]]
-			else:
-				candidates = Editor.database.select_words_by_pinyin_list_all(pinyinlist)
-				if candidates:
-					p = list (candidates[0]);
-					p[YLEN] = len (pinyinlist)
-					p[PHRASE] = p[PHRASE][:p[YLEN]]
-					return [p]
+				p = list (candidates[0]);
+				p[YLEN] = len (pinyinlist)
+				p[PHRASE] = p[PHRASE][:p[YLEN]]
+				return [p]
 		max_freq = 0
 		max_length =0
 		#~ print "try words"
 		#~ if longest==1:
 			#~ return [Editor.database.select_words_by_pinyin_list(pinyinlist[:1])[0][PHRASE]]
 		#~ print longest
-		for i in range (1, longest + 1):
+		for i in range (1, len(pinyinlist)):
 			candidates = Editor.database.select_words_by_pinyin_list (pinyinlist[:i])
 			if not candidates:
 				continue
@@ -594,9 +674,13 @@ class Editor:
 		for i in range (0, len (phrase) ):
 			if i < 4:
 				self.wordlist[ self.cursor + i ].real_pinyin_id = candidate[ i + 1 ]
+				if self.enable_mohuyin:
+					self.wordlist[ self.cursor + i ].mohuyin()
 			else:
 				py = candidate[YX].split("'")
 				self.wordlist[ self.cursor + i ].real_pinyin_id = PINYIN_DICT[ py[ i - 4 ] ]
+				if self.enable_mohuyin:
+					self.wordlist[ self.cursor + i ].mohuyin()
 			self.wordlist[ self.cursor + i ].set_char( phrase[i] )
 			self.wordlist[ self.cursor + i ].manual = True
 		self.cursor += len (phrase)
@@ -604,23 +688,6 @@ class Editor:
 			self.reparse (self.cursor);
 		self.update ()
 
-	def select (self, n):
-		if self.page + n >= len(self.candidates):
-			raise InputException()
-		candidate = self.candidates[self.page+n]
-		if self.pinyinlist:
-			self.pinyin_select(candidate, True)
-		else:
-			self.wordlist_manual_select(candidate)
-
-	def select_cursor (self):
-		candidate = self.candidates[self.candidate_cursor]
-		#~ print candidate[PHRASE]
-		if self.pinyinlist:
-			self.pinyin_select(candidate, True)
-		else:
-			self.wordlist_manual_select(candidate)
-		
 	def commit (self):
 		if self.pinyinlist:
 			self.convert_all ()
@@ -681,59 +748,108 @@ class Editor:
 			self.cursor = pos - 1
 		self.update ()
 
+	def select (self, n):
+		#~ print self.lookup_table.get_current_page_size()
+		if n >= self.lookup_table.get_current_page_size():
+			raise InputException()
+		candidate = self.candidates[self.lookup_table.get_current_page_start()+n]
+		if self.pinyinlist:
+			self.pinyin_select(candidate, True)
+		else:
+			self.wordlist_manual_select(candidate)
+
+	def select_cursor (self):
+		candidate = self.candidates[self.lookup_table.get_cursor_pos()]
+		#~ print candidate[PHRASE]
+		if self.pinyinlist:
+			self.pinyin_select(candidate, True)
+		else:
+			self.wordlist_manual_select(candidate)
+
+	def recursive_mohuyin_pinyinlist(self, pinyinlist):
+		for i in self.mohuyin(pinyinlist[0].get_screen_pinyin()):
+			if pinyinlist[1:]:
+				for p in self.recursive_mohuyin_pinyinlist(pinyinlist[1:]):
+					yield [PinYinWord(pinyin = i)] + p
+			else:
+				yield [PinYinWord(pinyin = i)]
+
+	def recursive_mohuyin(self, strl):
+		for i in self.mohuyin(strl[0]):
+			if strl[1:]:
+				for p in self.recursive_mohu(strl[1:]):
+					yield [i] + p
+			else:
+				yield [i]
+	
+	def mohuyin(self, pinyin):
+		#~ print pinyin
+		if pinyin[0] == "'":
+			spliter = "'"
+			pinyin = pinyin[1:]
+		else:
+			spliter = ""
+		if pinyin[:2] in SHENGMU_LIST:
+			shengmu = pinyin[:2]
+			yunmu = pinyin[2:]
+		elif pinyin[:1] in SHENGMU_LIST:
+			shengmu = pinyin[:1]
+			yunmu = pinyin[1:]
+		else:
+			shengmu = ""
+			yunmu = pinyin
+		if shengmu in self.shengmu_mohu:
+			shengmu = self.shengmu_mohu[shengmu]
+		else:
+			shengmu = [shengmu]
+		if yunmu in self.yunmu_mohu:
+			yunmu = self.yunmu_mohu[yunmu]
+		else:
+			yunmu = [yunmu]
+		if pinyin in PINYIN_PARTIAL_LIST:
+			for q in yunmu:
+				if i + q in SHENGMU_LIST or i + q in PINYIN_LIST or i + q in PINYIN_PARTIAL_LIST:
+					yield spliter + i + q
+		else:
+			for i in shengmu:
+				for q in yunmu:
+					if i + q in SHENGMU_LIST or i + q in PINYIN_LIST:
+						yield spliter + i + q
+	def parsr_mohuyin(self, pinyinlist):
+		candidates = []
+		if self.enable_mohuyin:
+			ss = Set()
+			for p in self.recursive_mohuyin_pinyinlist(pinyinlist):
+				#~ print u" ".join( i.get_pinyin() for i in p) 
+				for i in range (len (p), 0, -1):
+					ss.update(Editor.database.select_words_by_pinyin_list (p[:i]))
+			candidates = list(ss)
+			candidates.sort(cmp = candidate_sort)
+		else:
+			for i in range (len (pinyinlist), 0, -1):
+				candidates += Editor.database.select_words_by_pinyin_list (pinyinlist[:i])
+		return candidates
+				
 	def update_candidates (self):
 		if self.is_empty():
 			self.candidates = []
 		elif self.pinyinlist:
-			self.candidates = []
-			for i in range (len (self.pinyinlist), 0, -1):
-				self.candidates += Editor.database.select_words_by_pinyin_list (self.pinyinlist[:i])
+			self.candidates = self.parsr_mohuyin(self.pinyinlist)
 		elif len(self.wordlist)>self.cursor:
-			self.candidates = []
-			for i in range (len (self.wordlist),self.cursor, -1):
-				self.candidates += Editor.database.select_words_by_pinyin_list (self.wordlist[self.cursor:i])
+			self.candidates = self.parsr_mohuyin(self.wordlist[self.cursor:])
 		else:
 			self.candidates = []
-		self.page = 0
+		self.update_lookup_table()
 
-	def get_lookup_table (self):
-		if len (self.candidates)-self.page>9:
-			return [(c[PHRASE], c[ADJ_FREQ] == 0 or c[ADJ_FREQ]%USER_PHRASE and c[ADJ_FREQ]%USER_WORD) for c in self.candidates[self.page:self.page+10]]
-		else:
-			return [(c[PHRASE], c[ADJ_FREQ] == 0 or c[ADJ_FREQ]%USER_PHRASE and c[ADJ_FREQ]%USER_WORD) for c in self.candidates[self.page:]]
-
-	def get_candidate_cursor(self):
-		if self.candidate_cursor == None:
-			return None
-		else:
-			return self.candidate_cursor - self.page
-
-	def move_candidate_cursor(self,num):
-		if not self.candidates:
-			raise InputException()
-		if self.candidate_cursor == None:
-			self.candidate_cursor = self.page
-		else:
-			self.candidate_cursor += num
-			if self.candidate_cursor >= len (self.candidates):
-				self.candidate_cursor = 0
-			elif self.candidate_cursor<0:
-				self.candidate_cursor = 0
-				raise InputException()
-			self.page = self.candidate_cursor /9 * 9
-
-	def turn_page (self, num):
-		if not self.candidates:
-			raise InputException()
-		if num>0:
-			if	num+self.page < len (self.candidates):
-				self.page += num
+	def update_lookup_table (self):
+		self.lookup_table.clear()
+		self.lookup_table.show_cursor(False)
+		for c in self.candidates:
+			if  c[ADJ_FREQ] == 0 or c[ADJ_FREQ]%USER_PHRASE and c[ADJ_FREQ]%USER_WORD:
+				self.lookup_table.append_candidate (c[PHRASE])
 			else:
-				self.page =0
-		else:
-			if self.page >= -num:
-				self.page += num
-		self.candidate_cursor = self.page
+				attrs = [scim.Attribute (0, len(c[PHRASE]), scim.ATTR_FOREGROUND, RGB (0, 0, 0xef))]
+				self.lookup_table.append_candidate (c[PHRASE], attrs)
 
 class Engine (IMEngine):
 	def __init__ (self, factory, config, encoding, id):
@@ -746,35 +862,36 @@ class Engine (IMEngine):
 		self.reload_config(config)
 		self._log = scim.Log.Log ("ZhengJu")
 		#~ print "init"
-
+	def clear(self):
+		pass
 	def reset(self):
 		#~ print "reset"
 		if self._editor.wordlist:
 			self.commit_string (self._editor.commit())
 		else:
 			self._editor.clear()
+		self.clear()
 		self.origin_string = None
 		self._double_quotation_state = False
 		self._single_quotation_state = False
 		self._prev_key = None
-		self._shift_key = None
+		self._shift_key = None	
+		self.pipe = None	
 		self.update ()	
 		props = [self._status_property, self._setup_property]
 		self.register_properties (props)
 		self.update_properties ()
-		self.pipe = None
 	def update_preedit (self):
-		string = self._editor.get_preedit ()
+		string = self._editor.get_preedit () + self.get_extra_string()
 		if (string == u""):
 			self.hide_preedit_string ()
 		else:
 			self.show_preedit_string ()
-			self.update_preedit_string (string, [])
+			self.update_preedit_string (string , [])
 			self.update_preedit_caret (self._editor.get_screen_cursor())
 
 	def focus_out(self):
 		#~ print "out reset"
-
 		self.reset()
 		IMEngine.focus_out (self)
 		
@@ -790,25 +907,11 @@ class Engine (IMEngine):
 			self.start_helper ("61af6de6-c29d-421e-9e1b-e34a29c68c76")
 
 	def update_candidate (self):
-		candidates = self._editor.get_lookup_table()
-		if len (candidates) == 0:
-			self.hide_lookup_table ()
-		else:
-			#~ self.hide_lookup_table ()
-			self._lookup_table.clear ()
-			for c in candidates:
-				if c[1]:
-					self._lookup_table.append_candidate (c[0])
-				else:
-					attrs = [scim.Attribute (0, len(c[0]), scim.ATTR_FOREGROUND, RGB (0, 0, 0xef))]
-					self._lookup_table.append_candidate (c[0], attrs)
-			if self._editor.get_candidate_cursor() != None:
-				self._lookup_table.show_cursor (True)
-				self._lookup_table.set_cursor_pos (self._editor.get_candidate_cursor())
-			else:
-				self._lookup_table.show_cursor (False)
-			self.update_lookup_table (self._lookup_table)
+		if self._editor.candidates:
+			self.update_lookup_table(self._editor.lookup_table)
 			self.show_lookup_table()
+		else:
+			self.hide_lookup_table ()
 
 	def update_aux(self):
 		if self._editor.predict:
@@ -843,9 +946,19 @@ class Engine (IMEngine):
 		self._editor.load_config(config)
 		self.progresivepromp = config.read ("/IMEngine/Python/ZhengJu/ProgressivePrompt", False)
 
+	def lookup_table_page_down (self):
+		self._editor.lookup_table.page_down ();
+		self.update()
+		return True
+
+	def lookup_table_page_up (self):
+		self._editor.lookup_table.page_up ();
+		self.update()
+		return True
+		
 	def process_key_event (self, key):
 		#~ print key.code
-		if self._chinese_mode and self._editor.is_end():
+		if self._chinese_mode and self._editor.is_end() and not self.get_extra_string():
 			if (key.code == KeyCode.KEY_Shift_L or key.code == KeyCode.KEY_Shift_R)	\
 				and key.mask & KeyMask.ReleaseMask:
 				if self._shift_key:
@@ -854,7 +967,7 @@ class Engine (IMEngine):
 			if key.mask == KeyMask.ShiftMask and (key.code >= KeyCode.KEY_A and key.code <= KeyCode.KEY_Z) \
 				or key.mask & KeyMask.CapsLockMask:
 				self.change_mode()
-		else:
+		elif not self.get_extra_string():
 			if (key.code == KeyCode.KEY_Shift_L or key.code == KeyCode.KEY_Shift_R)\
 				and key.mask & KeyMask.ReleaseMask:
 				if self._shift_key:
@@ -907,7 +1020,7 @@ class Engine (IMEngine):
 		return False
 
 	def chinese_process_key_event (self, key):
-		if self._editor.is_empty ():
+		if self._editor.is_empty() and not self.get_extra_string():
 			if key.code <= 127 and ascii.ispunct (chr (key.code)):
 				self.commit_string (self.convert_to_full_width (unichr (key.code)))
 				return True
@@ -917,12 +1030,13 @@ class Engine (IMEngine):
 				return True
 			else:
 				return False
+		#~ print unichr (key.code)
 		if key.code in (KeyCode.KEY_Control_L,KeyCode.KEY_Control_R,
 			KeyCode.KEY_Alt_L, KeyCode.KEY_Alt_R):
 			return True
 		elif key.code in (KeyCode.KEY_KP_Space, KeyCode.KEY_space):
 			#~ print self._editor.get_candidate_cursor()
-			if self._editor.get_candidate_cursor() != None:
+			if self._editor.candidates and self._editor.lookup_table.is_cursor_visible():
 				self._editor.select_cursor()
 				return True
 			elif self._editor.pinyinlist:
@@ -935,13 +1049,15 @@ class Engine (IMEngine):
 				self.commit_string (self._editor.commit())
 				return True
 		elif key.code == KeyCode.KEY_BackSpace:
+			if not self._editor.pinyinlist and self.get_extra_string():
+				raise InputException()
 			self._editor.del_current()
 			return True
 		elif key.code == KeyCode.KEY_Delete:
-			if self._editor.candidate_cursor == None:
-				self._editor.del_next ()
-			else:
+			if self._editor.lookup_table.is_cursor_visible():
 				self._editor.delete_cursor_phrase ()
+			else:
+				self._editor.del_next ()
 			return True
 		elif key.code >= KeyCode.KEY_0 and key.code <= KeyCode.KEY_9 and key.mask & KeyMask.ControlMask:
 			self._editor.delete_phrase (key.code - KeyCode.KEY_1)
@@ -966,49 +1082,72 @@ class Engine (IMEngine):
 				self._shift_key = None
 			return True
 		elif key.code in (KeyCode.KEY_equal, KeyCode.KEY_bracketright, KeyCode.KEY_Page_Down):
-			self._editor.turn_page (9)
-			return True
+			if self._editor.candidates:
+				self._editor.lookup_table.page_down ();
+				return True
+			else:
+				raise InputException()				
 		elif key.code in (KeyCode.KEY_minus, KeyCode.KEY_bracketleft, KeyCode.KEY_Page_Up):
-			self._editor.turn_page (-9)
-			return True
+			if self._editor.candidates:
+				self._editor.lookup_table.page_up ();
+				return True
+			else:
+				raise InputException()
 		elif key.code==KeyCode.KEY_Up:
-			self._editor.move_candidate_cursor(-1)
-			return True
+			if self._editor.candidates:
+				self._editor.lookup_table.cursor_up()
+				self._editor.lookup_table.show_cursor(True)
+				return True
+			else:
+				raise InputException()
 		elif key.code==KeyCode.KEY_Down:
-			self._editor.move_candidate_cursor(1)
-			return True
+			if self._editor.candidates:
+				self._editor.lookup_table.cursor_down()
+				self._editor.lookup_table.show_cursor(True)
+				return True
+			else:
+				raise InputException()
 		elif key.code == KeyCode.KEY_Left or key.code == KeyCode.KEY_b and key.mask & KeyMask.ControlMask:
 			self._editor.move_cursor (-1)
 			return True
 		elif key.code == KeyCode.KEY_Right or key.code == KeyCode.KEY_f and key.mask & KeyMask.ControlMask:
+			if self.get_extra_string():
+				raise InputException()
 			self._editor.move_cursor (1)
 			return True
 		elif key.code == KeyCode.KEY_h and key.mask & KeyMask.ControlMask or key.code == KeyCode.KEY_Home:
+			if self.get_extra_string():
+				raise InputException()
 			self._editor.move_cursor_to (1)
 			return True
 		elif key.code == KeyCode.KEY_e and key.mask & KeyMask.ControlMask or key.code == KeyCode.KEY_End:
+			if self.get_extra_string():
+				raise InputException()
 			self._editor.move_cursor_to (0)
 			return True
 		elif key.code in (KeyCode.KEY_Return, KeyCode.KEY_KP_Enter):
-			self.commit_string (self._editor.commit())
+			self.commit_string (self._editor.commit() + self.get_extra_string())
+			self.clear()
 			return True
 		elif key.code == KeyCode.KEY_Escape or key.code == KeyCode.KEY_c and key.mask & KeyMask.ControlMask:
 			if self.origin_string:
 				self.commit_string(self.origin_string)
 				self._editor.clear()
 				self.origin_string = None
-			if self._editor.get_candidate_cursor() != None:
+			elif self._editor.lookup_table.is_cursor_visible():
+				self._editor.lookup_table.show_cursor(False)
 				self._editor.update()
 			else:
+				self.clear()
 				self._editor.clear()
 			return True
-		elif key.code <= 127 and ascii.ispunct (chr (key.code)):
+		elif key.code <= 127 and ascii.ispunct (chr (key.code)) and not self.get_extra_string():
 			if not self._editor.is_empty ():
 				self.commit_string (self._editor.commit ())
 			self.commit_string (self.convert_to_full_width (unichr (key.code)))
 			return True
 		else:
-			raise Exception ()
+			raise InputException ()
 
 	def convert_to_full_width (self, c):
 		if c == u".":
