@@ -413,11 +413,8 @@ class XMSQLiteDB:
 		for i in range(_len):
 			_condition += 'AND m%d = ? ' % i
 		# you can increase the x in _len + x to include more result, but in the most case, we only need one more key result, so we don't need the extra overhead :)
-		# if len == 1 the x will be 2, if len > 1, the x will be 3
-		if _len > 1:
-			x_len = 3
-		else:
-			x_len = 2
+		# we just search for 1 key more, if nothing, then 2 key more
+		x_len = 2
 		sqlstr = '''SELECT * FROM (SELECT * FROM main.phrases WHERE mlen < %(mk)d  %(condition)s 
 		UNION ALL
 		SELECT * FROM user_db.phrases WHERE mlen < %(mk)d %(condition)s )
@@ -427,6 +424,16 @@ class XMSQLiteDB:
 		_xms += _xms
 		result = self.db.execute(sqlstr, _xms).fetchall()
 		self.db.commit()
+		# we do this only in some case, the 3 key have nothing but 4 does
+		if len(result) ==0:
+			x_len = 3
+			sqlstr = '''SELECT * FROM (SELECT * FROM main.phrases WHERE mlen < %(mk)d  %(condition)s 
+			UNION ALL
+			SELECT * FROM user_db.phrases WHERE mlen < %(mk)d %(condition)s )
+			ORDER BY mlen ASC, user_freq DESC, freq DESC;''' % { 'mk':_len+x_len, 'condition':_condition}
+			result = self.db.execute(sqlstr, _xms).fetchall()
+			self.db.commit()
+		
 		sysdb={}
 		usrdb={}
 		_cand = []
@@ -446,7 +453,6 @@ class XMSQLiteDB:
 				_cand.append( sysdb[key][0] + sysdb[key][1] )
 		
 		_cand.sort(cmp=self.compare)
-		
 		return _cand[:]
 
 	def select_zi( self, xms ):
