@@ -125,7 +125,8 @@ class Editor:
 		elif self._chars[1]:
 			self._chars[1].append (c)
 		else:
-			if c in self._valid_input_chars:
+			if (not self._py_mode and ( c in self._valid_input_chars)) or\
+				(self._py_mode and (c in u'abcdefghijklmnopqrstuvwxyz')):
 				try:
 					self._xmkey_list += self._parser (c)
 					self._chars[0].append (c)
@@ -1020,7 +1021,7 @@ class XingMaEngine (IMEngine):
 		if self._editor.is_empty ():
 			# we have not input anything
 			if key.code <= 127 and (not key.mask & KeyMask.AltMask + KeyMask.ControlMask):
-				if ascii.ispunct (key.code) and ( unichr(key.code) not in self._valid_input_chars ):
+				if ascii.ispunct (key.code) and ( unichr(key.code) not in self._valid_input_chars ) :
 					self.commit_string (cond_punct_translate (unichr (key.code)))
 					return True
 				if ascii.isdigit (key.code):
@@ -1154,7 +1155,9 @@ class XingMaEngine (IMEngine):
 		elif key.mask & KeyMask.AltMask:
 			return False
 
-		elif unichr(key.code) in self._valid_input_chars:
+		elif unichr(key.code) in self._valid_input_chars or \
+				( self._editor._py_mode and \
+					unichr(key.code) in u'abcdefghijklmnopqrstuvwxyz' ):
 			res = self._editor.add_input ( unichr(key.code) )
 			if not res:
 				if ascii.ispunct (key.code):
