@@ -86,8 +86,7 @@ PyIMEngine::operator delete (void *p)
 	PyObject *pArgs = NULL;							\
 	pFunc = PyObject_GetAttrString (this->self, fun);\
 	if (pFunc != NULL) {							\
-		pArgs = PyTuple_New (1);					\
-		PyTuple_SetItem (pArgs, 0, arg1);			\
+		pArgs = Py_BuildValue ("(O)", arg1);		\
 		pValue = PyObject_CallObject (pFunc, pArgs);\
 		Py_DECREF (pFunc);							\
 		Py_DECREF (pArgs);							\
